using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.FormControls;
using CMS.PortalEngine;
using CMS.DataEngine;
using CMS.TreeEngine;

using TreeNode = CMS.TreeEngine.TreeNode;
using CMS.SiteProvider;

public partial class CMSModules_Content_FormControls_Tags_TagSelector : FormEngineUserControl
{
    #region "Variables"

    private bool mEnabled = true;

    #endregion

    #region "Properties"

    /// <summary>
    /// Enable/disable control.
    /// </summary>
    public override bool Enabled
    {
        get
        {
            return this.mEnabled;
        }
        set
        {
            this.mEnabled = value;
            btnSelect.Enabled = value;
        }
    }


    /// <summary>
    /// Gets or sets the field value.
    /// </summary>
    public override object Value
    {
        get
        {
            return TagHelper.GetTagsForSave(this.txtTags.Text.Trim());
        }
        set
        {
            this.txtTags.Text = ValidationHelper.GetString(value, "");
        }
    }


    /// <summary>
    /// Tag Group ID.
    /// </summary>
    public int GroupId
    {
        get
        {
            int mGroupId = ValidationHelper.GetInteger(this.GetValue("GroupID"), 0);
            if ((mGroupId == 0) && (this.Form != null))
            {
                string path = "";

                // When inserting new document
                if (this.Form.ParentObject != null)
                {
                    // Get path and groupID of the parent node
                    path = ((TreeNode)this.Form.ParentObject).NodeAliasPath;
                    mGroupId = ((TreeNode)this.Form.ParentObject).DocumentTagGroupID;
                    // If nothing found try get inherited value
                    if (mGroupId == 0)
                    {
                        mGroupId = ValidationHelper.GetInteger(((TreeNode)this.Form.ParentObject).GetInheritedValue("DocumentTagGroupID", true), 0);
                    }
                }
                // When editing existing document
                else if (this.Form.EditedObject != null)
                {
                    // Get path and groupID of the parent node
                    path = ((TreeNode)this.Form.EditedObject).NodeAliasPath;
                    mGroupId = ((TreeNode)this.Form.EditedObject).DocumentTagGroupID;
                    // If nothing found try get inherited value
                    if (mGroupId == 0)
                    {
                        mGroupId = ValidationHelper.GetInteger(((TreeNode)this.Form.EditedObject).GetInheritedValue("DocumentTagGroupID", true), 0);
                    }
                }
            }

            return mGroupId;
        }
        set
        {
            this.SetValue("GroupID", value);
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        // Ensure Script Manager is the first control on the page
        ScriptManager sMgr = ScriptManager.GetCurrent(Page);
        if (sMgr != null)
        {
            sMgr.Services.Add(new ServiceReference("~/CMSModules/Content/FormControls/Tags/TagSelectorService.asmx"));
        }

        // Register the dialog script
        ScriptHelper.RegisterDialogScript(this.Page);

        // Register tag script 
        ScriptHelper.RegisterStartupScript(this, typeof(string), "tagScript", ScriptHelper.GetScript(tagScript()));

        // Create script for valid inserting into textbox
        ScriptHelper.RegisterStartupScript(this, typeof(string), "tagSelectScript", ScriptHelper.GetScript(@"
function itemSelected(source, eventArgs) {
    var txtBox = source.get_element();
    if (txtBox) {
        txtBox.value = eventArgs.get_text().replace(/\'""/,'""').replace(/""\'/,'""');
    }
}
function resetPosition(object, args) {
    var tb = object._element;
    var tbposition = findPosition(tb);

    var xposition = tbposition[0];
    var yposition = tbposition[1] + 22; // 22 = textbox height + a few pixels spacing

    var ex = object._completionListElement;
    if (ex) {
        $common.setLocation(ex, new Sys.UI.Point(xposition, yposition));
    }
}
function findPosition(obj) {
    var posX = 0, posY = 0;
    if (typeof (obj.offsetParent) != 'undefined') {
        posX += obj.offsetLeft;
        posY += obj.offsetTop;
        while (obj) {
            if ((obj.style.position != 'absolute') && (obj.style.position != 'relative') && (obj.style.position != 'fixed')) {
                posX += obj.offsetLeft;
                posY += obj.offsetTop;
                obj = obj.offsetParent;
            }
            else {
                obj = null;
            }
        }
    }
    else {
        posX = obj.x;
        posY = obj.y;
    }
    return [posX, posY];
}"));

        this.btnSelect.Text = ResHelper.GetString("general.select");
    }


    protected void Page_PreRender(object sender, EventArgs e)
    {
        // Enable / Disable control
        txtTags.Enabled = Enabled;
        btnSelect.Enabled = Enabled;
        if (Enabled)
        {
            autoComplete.ContextKey = GroupId.ToString();
            btnSelect.OnClientClick = String.Format("tagSelect('{0}','{1}'); return false;", txtTags.ClientID, GroupId);
        }
    }


    /// <summary>
    /// Returns tag JS script.
    /// </summary>
    private string tagScript()
    {
        string baseUrl = IsLiveSite ? "~/CMSFormControls/LiveSelectors/TagSelector.aspx" : "~/CMSFormControls/Selectors/TagSelector.aspx";

        // Build script with modal dialog opener and set textbox functions
        return String.Format(@"
function tagSelect(id,group){{    
    var textbox = document.getElementById(id);
    if (textbox != null){{
        var tags = encodeURIComponent(textbox.value.replace(/\|/,""-"").replace(/%/,""-"")).replace(/&/,""%26"");
        modalDialog('{0}?textbox='+ id +'&group='+ group +'&tags=' + tags, 'TagSelector', 570, 670);
    }}
}}
function setTagsToTextBox(textBoxId,tagString){{
    if (textBoxId != '') {{
        var textbox = document.getElementById(textBoxId);
        if (textbox != null){{
            textbox.value = decodeURI(tagString);
        }}
    }}
}}", ResolveUrl(baseUrl));
    }
}
